
# q-expansion of newform 169.4.e.g, downloaded from the LMFDB on 25 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 169
weight = 4
poly_data = [256, 0, -144, 0, 65, 0, -9, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-1296, 0, 585, 0, -65, 0, 9, 0], 1040], [[0, -181, 0, 0, 0, 0, 0, -1], 260], [[116, 0, 0, 0, 0, 0, 1, 0], 65], [[4176, 0, -1885, 0, 325, 0, -29, 0], 1040], [[0, -256, 0, 585, 0, -65, 0, 9], 1040], [[0, -256, 0, 377, 0, -65, 0, 9], 832]]

hecke_ring_character_values = [[2, [0, 0, -1, 0, 0, 0, 0, 0]]]
aps_data = [[0, -1, 0, -2, 0, 0, 1, -2], [-1, 0, -4, 0, 3, -3, 0, 0], [0, 0, 0, 0, 0, 0, 5, 10], [0, 1, 0, 7, 0, 0, 0, 0], [0, -15, 0, -1, 0, 0, 15, -1], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -27, 0, 0, 16, 0, 0], [0, -5, 0, 73, 0, 0, 0, 0], [89, 0, 56, 0, 33, -33, 0, 0], [13, 0, -47, 0, 60, -60, 0, 0], [0, 0, 0, 0, 0, 0, 100, 120], [0, 44, 0, 73, 0, 0, -44, 73], [0, 20, 0, 259, 0, 0, -20, 259], [0, 0, -276, 0, 0, -97, 0, 0], [0, 0, 0, 0, 0, 0, 140, 100], [190, 0, 0, 0, -165, 0, 0, 0], [0, -55, 0, -377, 0, 0, 0, 0], [0, 0, 151, 0, 0, -200, 0, 0], [0, 91, 0, -283, 0, 0, -91, -283], [0, -105, 0, -11, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -85, 250], [140, 0, 0, 0, 40, 0, 0, 0], [0, 0, 0, 0, 0, 0, -100, -180], [0, 125, 0, -523, 0, 0, -125, -523], [0, 469, 0, -27, 0, 0, 0, 0], [151, 0, 471, 0, -320, 320, 0, 0], [1080, 0, 0, 0, -320, 0, 0, 0], [367, 0, 968, 0, -601, 601, 0, 0], [0, 0, 0, 0, 0, 0, -260, 230], [0, 0, 271, 0, 0, -388, 0, 0], [1853, 0, 1372, 0, 481, -481, 0, 0], [820, 0, 0, 0, 840, 0, 0, 0], [0, -16, 0, 663, 0, 0, 0, 0], [0, 0, -52, 0, 0, 245, 0, 0], [0, -40, 0, 1913, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 540, 960], [254, 0, 0, 0, -417, 0, 0, 0], [0, -1103, 0, -781, 0, 0, 0, 0], [0, 1721, 0, -993, 0, 0, -1721, -993], [0, 0, -1994, 0, 0, -223, 0, 0], [473, 0, 28, 0, 445, -445, 0, 0], [970, 0, 0, 0, 105, 0, 0, 0], [0, 0, 1824, 0, 0, -335, 0, 0], [0, -352, 0, 1421, 0, 0, 352, 1421], [0, 1731, 0, -1313, 0, 0, -1731, -1313], [0, 0, 832, 0, 0, -1325, 0, 0], [111, 0, 1056, 0, -945, 945, 0, 0], [0, -33, 0, -1111, 0, 0, 33, -1111], [0, 1051, 0, -1823, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -1700, 330], [4162, 0, 0, 0, -276, 0, 0, 0], [0, 0, 0, 0, 0, 0, -300, -5576], [0, 2680, 0, -989, 0, 0, 0, 0], [0, 0, -5736, 0, 0, -45, 0, 0], [-1493, 0, -1377, 0, -116, 116, 0, 0], [3831, 0, 2244, 0, 1587, -1587, 0, 0], [0, 0, -2598, 0, 0, -45, 0, 0], [0, -695, 0, -3921, 0, 0, 695, -3921], [0, 0, 5897, 0, 0, 804, 0, 0], [0, 0, 0, 0, 0, 0, -725, -7990], [2601, 0, 2904, 0, -303, 303, 0, 0], [0, -3148, 0, 3359, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -2120, -300], [-5080, 0, 0, 0, 1120, 0, 0, 0], [-3450, 0, 0, 0, 1980, 0, 0, 0], [0, 0, 0, 0, 0, 0, -2355, 630], [0, 2815, 0, -3299, 0, 0, 0, 0], [-2690, 0, 0, 0, -1295, 0, 0, 0], [0, 0, -4368, 0, 0, 349, 0, 0], [0, -815, 0, -1323, 0, 0, 815, -1323], [0, -3552, 0, 3511, 0, 0, 3552, 3511], [0, 0, 0, 0, 0, 0, 1200, 5304], [807, 0, 3888, 0, -3081, 3081, 0, 0], [0, 0, 3019, 0, 0, 5088, 0, 0], [0, 5355, 0, 193, 0, 0, -5355, 193], [0, -1653, 0, 4729, 0, 0, 0, 0], [-3890, 0, 0, 0, -3135, 0, 0, 0], [0, -3189, 0, 6287, 0, 0, 0, 0], [0, 280, 0, -1139, 0, 0, -280, -1139], [0, 1020, 0, -5163, 0, 0, 0, 0], [-11373, 0, -11768, 0, 395, -395, 0, 0], [0, 0, 0, 0, 0, 0, 625, -8862], [0, 2455, 0, -10251, 0, 0, -2455, -10251], [0, 0, 4509, 0, 0, 2488, 0, 0], [7677, 0, 8652, 0, -975, 975, 0, 0], [-11260, 0, 0, 0, -2100, 0, 0, 0], [0, 4295, 0, -337, 0, 0, 0, 0], [0, -7016, 0, 3143, 0, 0, 7016, 3143], [0, 5860, 0, -5299, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1080, 14120], [6920, 0, 0, 0, 5220, 0, 0, 0], [0, 2225, 0, 3867, 0, 0, -2225, 3867], [0, 1111, 0, -7763, 0, 0, 0, 0], [-59, 0, 1116, 0, -1175, 1175, 0, 0], [0, 0, 0, 0, 0, 0, 6500, 3220], [0, 0, 2924, 0, 0, -1027, 0, 0], [0, 4540, 0, 9287, 0, 0, -4540, 9287], [6738, 0, 0, 0, 6575, 0, 0, 0], [6299, 0, 1056, 0, 5243, -5243, 0, 0], [0, 0, 0, 0, 0, 0, -7785, -3110], [5860, 0, 0, 0, 6020, 0, 0, 0], [0, 3004, 0, 15133, 0, 0, -3004, 15133], [0, 0, -16144, 0, 0, -3513, 0, 0], [-1253, 0, -4898, 0, 3645, -3645, 0, 0], [-5620, 0, 0, 0, -2880, 0, 0, 0], [0, 0, 0, 0, 0, 0, -4047, 3214], [0, -5001, 0, 11853, 0, 0, 5001, 11853], [0, 0, 0, 0, 0, 0, -81, 6522], [6380, 0, 0, 0, 1600, 0, 0, 0], [-14291, 0, -11211, 0, -3080, 3080, 0, 0], [0, 0, 8012, 0, 0, -3351, 0, 0], [0, 1492, 0, 18799, 0, 0, -1492, 18799], [0, -2876, 0, -14607, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 2840, 14420], [0, -9175, 0, 4619, 0, 0, 0, 0], [0, 0, 11189, 0, 0, -4840, 0, 0], [0, -3973, 0, 20949, 0, 0, 0, 0], [9997, 0, 12008, 0, -2011, 2011, 0, 0], [2229, 0, -1514, 0, 3743, -3743, 0, 0], [0, 0, 19612, 0, 0, 5585, 0, 0], [0, 6000, 0, -16479, 0, 0, -6000, -16479], [-1081, 0, 7531, 0, -8612, 8612, 0, 0], [19210, 0, 0, 0, -4580, 0, 0, 0], [0, -5967, 0, 20711, 0, 0, 0, 0], [0, -8305, 0, -12519, 0, 0, 8305, -12519], [14150, 0, 0, 0, 8220, 0, 0, 0], [0, 1720, 0, -6413, 0, 0, 0, 0], [0, 0, 10328, 0, 0, 1575, 0, 0], [-8920, 0, 0, 0, -5860, 0, 0, 0], [0, 0, 0, 0, 0, 0, -17855, -6890], [0, 2225, 0, -20157, 0, 0, -2225, -20157], [0, -8193, 0, 19709, 0, 0, 8193, 19709], [-3671, 0, -7316, 0, 3645, -3645, 0, 0], [-3747, 0, 1902, 0, -5649, 5649, 0, 0], [0, 5415, 0, -249, 0, 0, 0, 0], [0, 3255, 0, -16833, 0, 0, -3255, -16833], [0, -13907, 0, 1361, 0, 0, 0, 0], [0, 941, 0, -15053, 0, 0, 0, 0], [0, 0, -7418, 0, 0, -7471, 0, 0], [10573, 0, 1913, 0, 8660, -8660, 0, 0], [0, 0, 0, 0, 0, 0, -1700, -13968], [0, 2795, 0, -33641, 0, 0, -2795, -33641], [0, 0, -2576, 0, 0, -7197, 0, 0], [0, 0, 0, 0, 0, 0, 13728, -9236], [5843, 0, 13063, 0, -7220, 7220, 0, 0], [0, -4055, 0, -14293, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 5755, 4590], [18630, 0, 0, 0, 8065, 0, 0, 0], [-4820, 0, 0, 0, -4480, 0, 0, 0], [0, 0, 0, 0, 0, 0, 11516, -16192], [0, 3176, 0, -38583, 0, 0, 0, 0], [11691, 0, 19831, 0, -8140, 8140, 0, 0], [11140, 0, 0, 0, 15140, 0, 0, 0], [-46117, 0, -43768, 0, -2349, 2349, 0, 0], [-3143, 0, -16292, 0, 13149, -13149, 0, 0], [25760, 0, 0, 0, -3420, 0, 0, 0], [0, 0, 2028, 0, 0, 13765, 0, 0], [0, -160, 0, 24973, 0, 0, 0, 0], [-21350, 0, 0, 0, -3605, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1760, -10390], [0, -2499, 0, -1033, 0, 0, 2499, -1033], [0, 0, 9954, 0, 0, 8693, 0, 0], [0, 0, 0, 0, 0, 0, 10040, 2160], [0, 0, 15564, 0, 0, -10135, 0, 0], [0, 15716, 0, -7113, 0, 0, -15716, -7113], [0, 0, 0, 0, 0, 0, -8500, -26640], [-3069, 0, -6764, 0, 3695, -3695, 0, 0], [0, 0, -5897, 0, 0, 1496, 0, 0]]
